@@ -16,7 +16,7 @@ rpc_client = pyjsonrpc.HttpClient("http://localhost:8080", gzipped = True)
 
 try:
     print repr(rpc_client.call("add", 1, 2))
-except pyjsonrpc.JsonRpcError, err:
+except pyjsonrpc.JsonRpcError as err:
     print err.code
     print err.message
     print err.data
